(*

Set up the routines needed for simulating the model

*)

Simulate[NumOfPeople_,NumOfPeriods_,InitialSavings_] := Block[{},

HowManyToSimulate = NumOfPeople;

If[Round[NumOfPeople*ZeroProb] != NumOfPeople*ZeroProb,
  Print["You must choose a value for NumOfPeople such that there is a whole number experiencing zero-income events in any period."]];

NumOfPoints = 200;  (* Number of points to use for the approximating CDF function *)
PickNumOfPoints =   (* A list of the positions of the points that will be chosen for the approximating function *)
  Table[Round[i], {i, 1, HowManyToSimulate, (HowManyToSimulate-1)/(NumOfPoints-1)}];

(* Construct a list of values that corresponds to a uniform grid on the range [0,1] to construct the
   discrete approximation to a CDF 
 *)
CDFTable = Table[i/(HowManyToSimulate-1) //N, {i,0, HowManyToSimulate-1}]; (* 0 to 1 *)

ListOfxtStateVars   = {"xt","PermInc","TranShockDraws","PermShockDraws","CDFwtFunc","CDFxtFunc"};
ListOfstStateVars   = {"st"};
ListOfxtStatsToCalc = {"wt","CDFxt","CDFwt","CDFxt","MPC","cGrowth","y","InterestRate","EcgVal","EcgsqVal","cLevel"};
ListOfstStatsToCalc = {"CDFst"};
ListOfVarsToTakeMeanOf = {"ct","st","xt","wt","TranShockDraws","PermShockDraws","PermInc","MPC"};
ListOfVarsToMake = Union[ListOfxtStateVars,ListOfstStateVars,ListOfxtStatsToCalc,ListOfstStatsToCalc,ListOfVarsToTakeMeanOf];

(* Set up the needed variables and fill them with zeros *)
Do[
  ToExpression[ListOfVarsToMake[[i]]<>"List = {0};"];
  ToExpression[ListOfVarsToMake[[i]]<>"Mean = {0};"];  
,{i,Length[ListOfVarsToMake]}];

(* 
The model described in the paper does not allow for unemployment with nonzero income, but these simulation
programs are more general and allow for it (the option is simply turned off for the sims of the paper.
The variables below indicate how many unemployed and how many zero-income-event agents there should
be per period.
*)

NumOfZeroGuys  = ZeroProb*NumOfPeople;
NumOfUnempGuys = UnempProb*NumOfPeople;

Print["Constructing transitory shock distribution"];

MakeShocks = True;

If[(ZeroProb>0 && Length[TranShockDrawsZero]==NumOfPeople) (* If we have already constructed the distribution for a previous sim *)
   ,(* then *) 
   {TranShockDraws,TranShockDensity} = {TranShockDrawsZero,TranShockDensityZero};
   MakeShocks=False;
   ];

If[(UnempProb>0 && Length[TranShockDrawsUnemp]==NumOfPeople) (*If we have already constructed the distribution for a previous sim*)
   ,(* then *)
   {TranShockDraws,TranShockDensity} = {TranShockDrawsUnemp,TranShockDensityUnemp};
   MakeShocks=False;
   ];

If[MakeShocks==True,  (* If they're not already resident in memory, construct them *)

(* 

Construct here a set of points corresponding
to a discretization of the continuous probability distribution where each observation corresponds to an equal-probability
event.  Then rather than drawing random shocks from a random number generator, we will scramble the order of the points.
This guarantees that every portion of the distribution is equally represented in each period, yet from the standpoint
of any individual agent their outcomes are ex-ante random
*)
   {TranShockDraws,TranShockDensity} = DiscreteApproxToMeanOneLogNormal[sigmaTran,NumOfPeople-NumOfZeroGuys-NumOfUnempGuys];
   
   If[ZeroProb>0,
     (* then modify the shocks and probabilities to incorporate the zero risk *)
     TranShockDraws  = TranShockDraws/(1-ZeroProb);
     TranShockDraws  = Join[TranShockDraws,Table[0,{NumOfZeroGuys}]];
     TranShockDensity = TranShockDensity*(1-ZeroProb);
     TranShockDensity = Join[TranShockDensity,Table[ZeroProb/NumOfZeroGuys,{NumOfZeroGuys}]];
     {TranShockDrawsZero,TranShockDensityZero} = {TranShockDraws,TranShockDensity};
    ];

   If[UnempProb>0, (* then modify the shocks and probabilities to incorporate the unemployment spell risk *) 
     TranShockDraws   =   (TranShockDraws-EUnempWage)/(1-UnempProb);
     TranShockDraws   =   Join[TranShockDraws,Table[UnempWage,{NumOfUnempGuys}]];
     TranShockDensity = TranShockDensity*(1-UnempProb); 
     TranShockDensity = Prepend[TranShockDensity,Table[UnempProb/NumOfUnempGuys,{NumOfUnempGuys}]]; 
     {TranShockDrawsUnemp,TranShockDensityUnemp} = {TranShockDraws,TranShockDensity};  
   ];
];

Print["Constructing permanent shock distribution"];

If[Length[PermShockDraws] != NumOfPeople,
  {PermShockDraws,PermShockDensity} = DiscreteApproxToMeanOneLogNormal[sigmaPerm,NumOfPeople];
];

Print["Constructing Period 1 Data"];
LoopOverPeriods = 1;

TranShockDrawsList[[1]] = TranShockDraws[[RandomPermutation[NumOfPeople]]];
PermShockDrawsList[[1]] = PermShockDraws[[RandomPermutation[NumOfPeople]]];

wtList[[1]]     = Table[InitialSavings,{NumOfPeople}];
CDFwtList[[1]]  = wtList[[1]];
xtList[[1]]     = wtList[[1]]+TranShockDrawsList[[1]];
CDFxtList[[1]]  = Sort[xtList[[1]]];
ctList[[1]]     = Map[ctSplice[#,LifePosn]&,xtList[[1]]];
MPCList[[1]]    = Map[MPC[#,LifePosn]&,xtList[[1]]];
stList[[1]]     = Chop[xtList[[1]]-ctList[[1]]];
CDFstList[[1]]  = Sort[stList[[1]]];

PermIncList[[1]] = PermShockDrawsList[[1]];
wtMean[[1]]      = InitialSavings;
xtMean[[1]]      = Mean[xtList[[1]]];
ctMean[[1]]      = Mean[ctList[[1]]];
MPCMean[[1]]     = Mean[MPCList[[1]]];
stMean[[1]]      = Mean[stList[[1]]];
PermIncMean[[1]] = Mean[PermShockDrawsList[[1]]];

cGrowthList[[1]]= Table[0,{NumOfPeople}];
yList[[1]]      = PermIncList[[1]] TranShockDrawsList[[1]];
InterestRateList[[1]] = Table[R-1,{NumOfPeople}];
EcgValList[[1]]   = Map[EtLogCtp1OCt[#,LifePosn]&,xtList[[1]]];
EcgsqValList[[1]] = Map[EtLogCtp1OCtsq[#,LifePosn]&,xtList[[1]]];


TranShockDrawsMean[[1]] = 1;
PermShockDrawsMean[[1]] = 1;

KeepSimulating[NumOfPeriods-1];

]; (* End Simulate *)


KeepSimulating[NumOfRemainingPeriods_] := Block[{},

LoopOverPeriods = NumOfRemainingPeriods;

While[LoopOverPeriods > 0,

CurrentPeriod = Length[wtList]+1;

NumOfPeople = Length[TranShockDraws];

Print["Simulating Period ",CurrentPeriod];

TranShockDrawsList = Append[TranShockDrawsList,TranShockDraws[[RandomPermutation[NumOfPeople]]]];  (* Disorder the ordered shock lists *)
PermShockDrawsList = Append[PermShockDrawsList,PermShockDraws[[RandomPermutation[NumOfPeople]]]];

CurrentPeriod = Length[TranShockDrawsList];

wtList     = Append[wtList,(R/(G[[LifePosn]] PermShockDrawsList[[CurrentPeriod]]))stList[[CurrentPeriod-1]]];
CDFwtList  = Append[CDFwtList,Sort[wtList[[CurrentPeriod]]]];
xtList     = Append[xtList,(R/(G[[LifePosn]] PermShockDrawsList[[CurrentPeriod]]))stList[[CurrentPeriod-1]]+TranShockDrawsList[[CurrentPeriod]]];
CDFxtList  = Append[CDFxtList,Sort[xtList[[CurrentPeriod]]]];
ctList     = Append[ctList,Map[ctSplice[#,LifePosn]&,xtList[[CurrentPeriod]]]];
MPCList    = Append[MPCList,Map[MPC[#,LifePosn]&,xtList[[CurrentPeriod]]]];
stList     = Append[stList,xtList[[CurrentPeriod]]-ctList[[CurrentPeriod]]];
CDFstList  = Append[CDFstList,Sort[stList[[CurrentPeriod]]]];


PermIncList = Append[PermIncList,PermIncList[[CurrentPeriod-1]] PermShockDrawsList[[CurrentPeriod]] G[[CurrentPeriod]]];

(* Make a function that reflects a linear interpolation among a sample of PickNumOfPoints in the simulated distributions *)
CDFwtFuncList =Append[CDFwtFuncList,Interpolation[Transpose[{CDFTable,CDFwtList[[-1]]}][[PickNumOfPoints]],InterpolationOrder->1]];
CDFxtFuncList =Append[CDFxtFuncList,Interpolation[Transpose[{CDFTable,CDFxtList[[-1]]}][[PickNumOfPoints]],InterpolationOrder->1]];

cGrowthList   = Append[cGrowthList,Log[G[[CurrentPeriod]] PermShockDrawsList[[-1]] ctList[[-1]]/ctList[[-2]]]];
yList         = Append[yList,TranShockDrawsList[[-1]] PermIncList[[-1]]];
EcgValList    = Append[EcgValList  ,Map[EtLogCtp1OCt[#,LifePosn]&,xtList[[CurrentPeriod]]]];
EcgsqValList  = Append[EcgsqValList,Map[EtLogCtp1OCtsq[#,LifePosn]&,xtList[[CurrentPeriod]]]];
InterestRateList=Append[InterestRateList,InterestRateList[[-1]]];

Do[              (* Take the mean of the requisite set of variables *)
  ToExpression[  
    ListOfVarsToTakeMeanOf[[i]]<>"Mean = Append["<>
    ListOfVarsToTakeMeanOf[[i]]<>"Mean,Mean["<>
    ListOfVarsToTakeMeanOf[[i]]<>"List[["<>ToString[CurrentPeriod]<>"]]]];"
  ]
,{i,Length[ListOfVarsToTakeMeanOf]}
];

If[LowMem == True && LoopOverPeriods< NumOfRemainingPeriods-3,  (* If memory is scarce, dump some past data no longer needed *)
  wtList[[-3]] = CDFwtList[[-3]] = xtList[[-3]] = CDFxtList[[-3]] = MPCList[[-3]] = 
  stList[[-3]] = CDFstList[[-3]] = PermIncList[[-3]] = TranShockDrawsList[[-3]] = 
  PermShockDrawsList[[-3]] = ctList[[-3]] = 0
  ];

LoopOverPeriods--;

]; (* End While LoopOverPeriods *)
]; (* End KeepSimulating *)
